#!/usr/bin/env python

# pep8: ignore=E501
# pylint: disable-msg=C0301

import os
import sys
import shutil
from os.path import exists

sys.path.insert(0, '..')
from tests import Test, main
from tests.providers import prosody
from libmace import files, lsb_release
from libmace.files import Transaction


class ProviderProsodyTest(Test):

    def setUp(self):
        Test.setUp(self)
        files.uid_by_name = hash
        files.gid_by_name = hash
        self.override(lsb_release, 'distributor_id', lambda: 'Fedora')

    def test_enusre_DataPath(self):
        provider = prosody.Provider()
        tran = Transaction('.')

        tran.ensure_file('/etc/prosody/prosody.cfg.lua', 'modules_enabled = {\n"posix";}')
        self.assertRaises(RuntimeError, provider.ensure, None, tran)

        tran.ensure_file('/etc/prosody/prosody.cfg.lua', 'data_path="data"\nmodules_enabled = {\n"posix";}')
        provider.ensure(None, tran)
        assert not tran.empty
        assert exists('data')
        assert ('./data', hash('prosody'), hash('prosody')) in self.chown

    def test_enusre_PosixModule(self):
        provider = prosody.Provider()
        tran = Transaction('.')

        tran.ensure_file('/etc/prosody/prosody.cfg.lua', 'data_path="data"')
        self.assertRaises(RuntimeError, provider.ensure, None, tran)

        tran.ensure_file('/etc/prosody/prosody.cfg.lua', 'data_path="data"\nmodules_enabled = {\n"posix";}')
        provider.ensure(None, tran)
        assert not tran.empty

    def test_enusre_SingleLog(self):
        provider = prosody.Provider()
        tran = Transaction('.')
        tran.ensure_file('/etc/prosody/prosody.cfg.lua',
                'data_path="data"\nlog="log/foo"\nmodules_enabled = {\n"posix";}')
        provider.ensure(None, tran)
        assert not tran.empty
        assert exists('log')
        assert ('./log', hash('prosody'), hash('prosody')) in self.chown

    def test_enusre_Logs(self):
        provider = prosody.Provider()
        tran = Transaction('.')
        tran.ensure_file('/etc/prosody/prosody.cfg.lua',
                'data_path="data"\nlog={info="log/foo";err="log2/bar"}\nmodules_enabled = {\n"posix";}')
        provider.ensure(None, tran)
        assert not tran.empty
        assert exists('log')
        assert exists('log2')
        assert ('./log', hash('prosody'), hash('prosody')) in self.chown
        assert ('./log2', hash('prosody'), hash('prosody')) in self.chown

    def test_enusre_Certs(self):
        provider = prosody.Provider()
        tran = Transaction('.')

        tran.ensure_file('/etc/prosody/prosody.cfg.lua',
                'data_path="data"\nssl = {}\nmodules_enabled = {\n"posix";}')
        self.assertRaises(RuntimeError, provider.ensure, None, tran)

        tran.ensure_file('/etc/prosody/prosody.cfg.lua',
                'data_path="data"\nssl = {key=""}\nmodules_enabled = {\n"posix";}')
        self.assertRaises(RuntimeError, provider.ensure, None, tran)

        tran.ensure_file('/etc/prosody/prosody.cfg.lua',
                'data_path="data"\nssl = {certificate=""}\nmodules_enabled = {\n"posix";}')
        self.assertRaises(RuntimeError, provider.ensure, None, tran)

        tran.ensure_file('/etc/prosody/prosody.cfg.lua',
            'data_path="data"\nssl = {key=""; certificate="";}')
        self.assertRaises(RuntimeError, provider.ensure, None, tran)

        tran.ensure_file('/etc/prosody/prosody.cfg.lua',
            'data_path="data"\nssl = {key="certs/key"; certificate="certs/certificate";}\nmodules_enabled = {\n"posix";}')
        self.assertRaises(RuntimeError, provider.ensure, None, tran)

        tran.ensure_file('/etc/prosody/prosody.cfg.lua',
                'data_path="data"\nVirtualHost "example.org"\nssl = {key="certs/key"; certificate="certs/certificate";}\nmodules_enabled = {\n"posix";}')
        provider.ensure(None, tran)
        assert not tran.empty
        assert exists('certs')
        self.assertEqual(0700, os.stat('certs').st_mode & 0777)
        assert exists('certs/key')
        self.assertEqual(0400, os.stat('certs/key').st_mode & 0777)
        assert exists('certs/certificate')
        assert ('./certs', hash('prosody'), hash('prosody')) in self.chown
        assert ('./certs/key', hash('prosody'), hash('prosody')) in self.chown
        assert ('./certs/certificate', hash('prosody'), hash('prosody')) in self.chown

    def test_enusre_RecreateExpiredCert(self):
        provider = prosody.Provider()
        tran = Transaction('.')

        os.makedirs('certs')
        os.chmod('certs', 0700)
        shutil.copy('../../data/prosody.cert', 'certs/certificate')
        shutil.copy('../../data/prosody.key', 'certs/key')
        os.chmod('certs/key', 0400)
        orig_cert = file('certs/certificate').read()

        tran.ensure_file('/etc/prosody/prosody.cfg.lua',
                'data_path="data"\nVirtualHost "example.org"\nssl = {key="certs/key"; certificate="certs/certificate";}\nmodules_enabled = {\n"posix";}')
        provider.ensure(None, tran)
        assert not tran.empty
        assert orig_cert != file('certs/certificate').read()

    def test_confile_header(self):
        provider = prosody.Provider()
        assert 'pidfile = "/var/run/prosody.pid"' in provider.confpath_header(None, None)
        assert 'run_as_root = false' in provider.confpath_header(None, None)


if __name__ == '__main__':
    main()
