#!/usr/bin/env python

# pep8: ignore=E501
# pylint: disable-msg=C0301

import os
import sys
from os.path import exists, isdir, islink, lexists

sys.path.insert(0, '..')
import tests
from tests import Test, main
from libmace import files
from libmace.files import Transaction


class TransactionTest(Test):

    def test_ensure_file_ContentAbsentFile(self):
        tran = Transaction('.')
        self.assertRaises(RuntimeError, tran.ensure_file, 'foo')
        assert tran.empty
        assert not exists('foo')

    def test_ensure_file_ContentExistedFile(self):
        tran = Transaction('.')
        self.touch('foo')
        tran.ensure_file('foo')
        assert tran.empty
        tran.ensure_file('foo', content='bar')
        assert not tran.empty
        tran.rollback()
        self.assertEqual('foo', file('foo').read())

    def test_ensure_file_ContentNewContent(self):
        tran = Transaction('.')

        tran.ensure_file('foo', content='foo')
        assert not tran.empty
        self.assertEqual('foo', file('foo').read())

        tran.ensure_file('foo', content='bar')
        assert not tran.empty
        self.assertEqual('bar', file('foo').read())

        tran.rollback()
        assert not exists('foo')

    def test_ensure_file_AbsentFile(self):
        tran = Transaction('.')
        self.assertRaises(RuntimeError, tran.ensure_file, 'foo')
        assert tran.empty
        assert not exists('foo')

    def test_ensure_file_Mode(self):
        tran = Transaction('.')
        self.touch('foo')
        os.chmod('foo', 0777)

        tran.ensure_file('foo', mode=0777)
        assert tran.empty
        self.assertEqual(0777, os.stat('foo').st_mode & 0777)
        tran.rollback()
        self.assertEqual(0777, os.stat('foo').st_mode & 0777)

        tran.ensure_file('foo', mode=0400)
        assert not tran.empty
        self.assertEqual(0400, os.stat('foo').st_mode & 0777)
        tran.rollback()
        self.assertEqual(0777, os.stat('foo').st_mode & 0777)

    def test_ensure_file_Owner(self):
        tran = Transaction('.')
        self.touch('foo')

        stat = os.stat('.')
        tran.ensure_file('foo', owner=(stat.st_uid, stat.st_gid))
        assert tran.empty
        self.assertEqual([], self.chown)

        files.uid_by_name = lambda x: 1
        files.gid_by_name = lambda x: 1
        tran.ensure_file('foo', owner=('foo', 'bar'))

        assert not tran.empty
        # chown for backed up copy
        backup_path, uid, gid = self.chown.pop(0)
        self.assertEqual(stat.st_uid, uid)
        self.assertEqual(stat.st_gid, gid)
        path = backup_path.split(os.sep)
        self.assertEqual(3, len(path))
        self.assertEqual(['.', 'foo'], [path[0], path[-1]])
        # chown while ensuring
        path, uid, gid = self.chown.pop(0)
        self.assertEqual(1, uid)
        self.assertEqual(1, gid)
        self.assertEqual('./foo', path)

        self.touch((backup_path, 'bar'))
        del self.chown[:]
        tran.rollback()
        assert tran.empty
        self.assertEqual([], self.chown)
        self.assertEqual('bar', file('foo').read())

    def test_ensure_dir_TryToOverwriteNewFile(self):
        self.touch('foo')
        tran = Transaction('.')
        tran.ensure_file('foo', content='bar')
        assert not tran.empty
        self.assertRaises(RuntimeError, tran.ensure_dir, 'foo')

    def test_ensure_dir_NonDirsOverwrite(self):
        self.touch('foo')
        tran = Transaction('.')
        tran.ensure_dir('foo')
        assert not tran.empty
        assert isdir('foo')
        tran.rollback()
        assert exists('foo')
        assert not isdir('foo')

    def test_ensure_dir_NewDir(self):
        tran = Transaction('.')
        tran.ensure_dir('foo')
        assert not tran.empty
        assert isdir('foo')
        tran.rollback()
        assert not exists('foo')

    def test_ensure_dir_Mode(self):
        os.makedirs('foo')
        os.chmod('foo', 0777)
        tran = Transaction('.')
        tran.ensure_dir('foo', mode=0777)
        assert isdir('foo')
        assert tran.empty
        tran.ensure_dir('foo', mode=0700)
        assert not tran.empty
        assert isdir('foo')
        self.assertEqual(0700, os.stat('foo').st_mode & 0777)
        tran.rollback()
        assert tran.empty
        assert isdir('foo')
        self.assertEqual(0777, os.stat('foo').st_mode & 0777)

    def test_ensure_dir_Owner(self):
        os.makedirs('foo')
        tran = Transaction('.')

        stat = os.stat('.')
        tran.ensure_dir('foo', owner=(stat.st_uid, stat.st_gid))
        assert tran.empty
        self.assertEqual([], self.chown)

        files.uid_by_name = lambda x: 1
        files.gid_by_name = lambda x: 1
        tran.ensure_dir('foo', owner=('foo', 'bar'))

        assert not tran.empty
        path, uid, gid = self.chown.pop(0)
        self.assertEqual(1, uid)
        self.assertEqual(1, gid)
        self.assertEqual('./foo', path)

        del self.chown[:]
        tran.rollback()
        assert tran.empty
        path, uid, gid = self.chown.pop(0)
        self.assertEqual(stat.st_uid, uid)
        self.assertEqual(stat.st_gid, gid)
        self.assertEqual('./foo', path)

    def test_EnsureAfterAdd(self):
        self.touch('foo')
        os.chmod('foo', 0700)
        tran = Transaction('.')

        tran.ensure_file('foo', content='bar')
        tran.ensure_file('foo', mode=0777)
        self.assertEqual('bar', file('foo').read())
        self.assertEqual(0777, os.stat('foo').st_mode & 0777)

        tran.rollback()
        self.assertEqual('foo', file('foo').read())
        self.assertEqual(0700, os.stat('foo').st_mode & 0777)

    def test_ensure_absence(self):
        tran = Transaction('.')

        tran.ensure_absence('foo')
        assert tran.empty

        self.touch('foo')
        tran.ensure_absence('foo')
        assert not tran.empty
        assert not exists('foo')

        tran.rollback()
        assert exists('foo')

    def test_ensure_absence_Abs(self):
        tran = Transaction('.')
        path = tests.tmpdir + '/foo'

        tran.ensure_absence(path, abs_path=True)
        assert tran.empty

        self.touch(path)
        tran.ensure_absence(path, abs_path=True)
        assert not tran.empty
        assert not exists(path)

        tran.rollback()
        assert exists(path)

    def test_ensure_symlink(self):
        self.touch('src')
        tran = Transaction('.')

        tran.ensure_symlink('src', 'dst')
        assert not tran.empty
        assert islink('dst')
        self.assertEqual('src', os.readlink('dst'))

        tran.rollback()
        assert not exists('dst')
        assert exists('src')

    def test_ensure_symlink_Relink(self):
        os.symlink('foo', 'dst')
        tran = Transaction('.')

        tran.ensure_symlink('src', 'dst')
        assert not tran.empty
        assert islink('dst')
        self.assertEqual('src', os.readlink('dst'))

        tran.rollback()
        assert lexists('dst')
        self.assertEqual('foo', os.readlink('dst'))

    def test_ensure_symlink_SupressFile(self):
        self.touch('dst')
        tran = Transaction('.')

        tran.ensure_symlink('src', 'dst')
        assert not tran.empty
        assert islink('dst')
        self.assertEqual('src', os.readlink('dst'))

        tran.rollback()
        assert lexists('dst')
        assert not islink('dst')

    def test_ensure_symlink_SupressDir(self):
        os.makedirs('dst')
        tran = Transaction('.')

        tran.ensure_symlink('src', 'dst')
        assert not tran.empty
        assert islink('dst')
        self.assertEqual('src', os.readlink('dst'))

        tran.rollback()
        assert lexists('dst')
        assert isdir('dst')

    def test_ensure_symlink_Abs(self):
        self.touch('src')
        dst = tests.tmpdir + '/dst'
        tran = Transaction('.')

        tran.ensure_symlink('src', dst, abs_path=True)
        assert not tran.empty
        assert islink('dst')
        self.assertEqual('src', os.readlink('dst'))

        tran.rollback()
        assert not exists('dst')
        assert exists('src')


if __name__ == '__main__':
    main()
